@@ -1,6 +1,6 @@
 /*
     libzint - the open source barcode library
-    Copyright (C) 2021-2023 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2021-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -31,7 +31,6 @@
 
 #include "testcommon.h"
 #include "../output.h"
-#include <locale.h>
 #ifdef _WIN32
 #include <windows.h>
 #include <direct.h>
@@ -39,13 +38,13 @@
 
 static void test_check_colour_options(const testCtx *const p_ctx) {
     struct item {
-        char *fgcolour;
-        char *bgcolour;
+        const char *fgcolour;
+        const char *bgcolour;
         int ret;
-        char *expected;
+        const char *expected;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { "FFFFFF", "000000", 0, "" },
         /*  1*/ { "ffffff", "ffffff", 0, "" },
         /*  2*/ { "77777777", "33333333", 0, "" },
@@ -58,16 +57,16 @@ static void test_check_colour_options(const testCtx *const p_ctx) {
         /*  9*/ { "100,,100,100", ",1,2,", 0, "" },
         /* 10*/ { "100,100,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "882: Malformed foreground CMYK colour (4 decimal numbers, comma-separated)" },
         /* 11*/ { "100,100,99,1001", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "883: Malformed foreground CMYK colour (3 digit maximum per number)" },
-        /* 12*/ { "101,100,100,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "884: Malformed foreground CMYK colour C (decimal 0-100 only)" },
-        /* 13*/ { "100,101,100,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "885: Malformed foreground CMYK colour M (decimal 0-100 only)" },
-        /* 14*/ { "100,100,101,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "886: Malformed foreground CMYK colour Y (decimal 0-100 only)" },
-        /* 15*/ { "100,100,100,101", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "887: Malformed foreground CMYK colour K (decimal 0-100 only)" },
+        /* 12*/ { "101,100,100,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "884: Malformed foreground CMYK colour C (decimal 0 to 100 only)" },
+        /* 13*/ { "100,101,100,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "885: Malformed foreground CMYK colour M (decimal 0 to 100 only)" },
+        /* 14*/ { "100,100,101,100", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "886: Malformed foreground CMYK colour Y (decimal 0 to 100 only)" },
+        /* 15*/ { "100,100,100,101", "0,1,2,3", ZINT_ERROR_INVALID_OPTION, "887: Malformed foreground CMYK colour K (decimal 0 to 100 only)" },
         /* 16*/ { "100,100,100,100", "0,1,", ZINT_ERROR_INVALID_OPTION, "882: Malformed background CMYK colour (4 decimal numbers, comma-separated)" },
         /* 17*/ { "100,100,100,100", "0,0123,3,4", ZINT_ERROR_INVALID_OPTION, "883: Malformed background CMYK colour (3 digit maximum per number)" },
-        /* 18*/ { "100,100,100,100", "0,1,2,101", ZINT_ERROR_INVALID_OPTION, "887: Malformed background CMYK colour K (decimal 0-100 only)" },
+        /* 18*/ { "100,100,100,100", "0,1,2,101", ZINT_ERROR_INVALID_OPTION, "887: Malformed background CMYK colour K (decimal 0 to 100 only)" },
         /* 19*/ { "100,100,100,100", "0,1,2,3,", ZINT_ERROR_INVALID_OPTION, "882: Malformed background CMYK colour (4 decimal numbers, comma-separated)" },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, ret;
     struct zint_symbol symbol;
 
@@ -91,16 +90,16 @@ static void test_check_colour_options(const testCtx *const p_ctx) {
 
 static void test_colour_get_rgb(const testCtx *const p_ctx) {
     struct item {
-        char *colour;
+        const char *colour;
         int ret;
         unsigned char red;
         unsigned char green;
         unsigned char blue;
         unsigned char alpha;
-        char *expected_cmyk;
+        const char *expected_cmyk;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { "FFFFFF", 0, 0xFF, 0xFF, 0xFF, 0xFF, "0,0,0,0" },
         /*  1*/ { "000000", 0, 0x00, 0x00, 0x00, 0xFF, "0,0,0,100" },
         /*  2*/ { "FEDCBA", 0, 0xFE, 0xDC, 0xBA, 0xFF, "0,13,27,0" },
@@ -114,7 +113,7 @@ static void test_colour_get_rgb(const testCtx *const p_ctx) {
         /* 10*/ { "80,30,60,0", 0, 0x33, 0xB3, 0x66, 0xFF, "72,0,43,30" },
         /* 11*/ { "50,50,50,50", 0, 0x40, 0x40, 0x40, 0xFF, "0,0,0,75" },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, ret;
 
     testStart("test_colour_get_rgb");
@@ -124,8 +123,8 @@ static void test_colour_get_rgb(const testCtx *const p_ctx) {
         unsigned char red = 0, green = 0, blue = 0, alpha = 0, rgb_alpha = 0;
         int cyan, magenta, yellow, black;
         int have_alpha;
-        char rgb[9];
-        char cmyk[16];
+        char rgb[64];
+        char cmyk[64];
 
         if (testContinue(p_ctx, i)) continue;
 
@@ -155,18 +154,18 @@ static void test_colour_get_rgb(const testCtx *const p_ctx) {
 
 static void test_colour_get_cmyk(const testCtx *const p_ctx) {
     struct item {
-        char *colour;
+        const char *colour;
         int ret;
         int cyan;
         int magenta;
         int yellow;
         int black;
         unsigned char alpha;
-        char *expected_rgb;
+        const char *expected_rgb;
         int ret_rgb;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { "80,30,60,0", 0, 80, 30, 60, 0, 0xFF, "33B366FF", 0 },
         /*  1*/ { "50,50,50,50", 0, 50, 50, 50, 50, 0xFF, "404040FF", 0 },
         /*  2*/ { "0,0,0,100", 0, 0, 0, 0, 100, 0xFF, "000000FF", 0 },
@@ -174,7 +173,7 @@ static void test_colour_get_cmyk(const testCtx *const p_ctx) {
         /*  4*/ { "123456", 1, 79, 40, 0, 44, 0xFF, "123456FF", 0 },
         /*  5*/ { "12345678", 2, 79, 40, 0, 44, 0x78, "12345678", 1 },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, ret;
 
     testStart("test_colour_get_cmyk");
@@ -182,7 +181,7 @@ static void test_colour_get_cmyk(const testCtx *const p_ctx) {
     for (i = 0; i < data_size; i++) {
         int cyan, magenta, yellow, black;
         unsigned char red, green, blue, alpha, rgb_alpha;
-        char rgb[9];
+        char rgb[16];
 
         if (testContinue(p_ctx, i)) continue;
 
@@ -250,7 +249,7 @@ static void test_set_whitespace_offsets(const testCtx *const p_ctx) {
         float expected_qz_right;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { BARCODE_CODE128, 1, 0, 0, 0, 0, 0, 0.0f, /*expected*/ 1.0f, 0.0f, 1.0f, 0.0f, 0.0f },
         /*  1*/ { BARCODE_CODE128, 2, 0, 0, 0, 0, 0, 1.0f, /*expected*/ 2.0f, 0.0f, 2.0f, 0.0f, 0.0f },
         /*  2*/ { BARCODE_CODE128, 2, 3, 0, 0, 0, 0, 1.0f, /*expected*/ 2.0f, 3.0f, 2.0f, 3.0f, 0.0f },
@@ -264,7 +263,7 @@ static void test_set_whitespace_offsets(const testCtx *const p_ctx) {
         /* 10*/ { BARCODE_CODE128, 2, 3, 1, BARCODE_BIND_TOP | BARCODE_BOX | BARCODE_BIND, 0, 0, 1.0f, /*expected*/ 3.0f, 4.0f, 3.0f, 3.0f, 0.0f }, /* BIND_TOP wins */
         /* 11*/ { BARCODE_CODE128, 2, 3, 1, BARCODE_BOX | BARCODE_QUIET_ZONES, 0, 0, 1.0f, /*expected*/ 13.0f, 4.0f, 13.0f, 4.0f, 10.0f },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i;
     struct zint_symbol symbol = {0};
     float xoffset, yoffset, roffset, boffset, qz_right;
@@ -324,13 +323,13 @@ static void test_set_whitespace_offsets(const testCtx *const p_ctx) {
 
 static void test_fopen(const testCtx *const p_ctx) {
     struct item {
-        char dir[32];
-        char subdir[32];
-        char *filename;
+        const char dir[32];
+        const char subdir[32];
+        const char *filename;
         int succeed;
     };
     /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
+    static const struct item data[] = {
         /*  0*/ { "", "", "out.png", 1 },
         /*  1*/ { "out_test", "", "out.png", 1 },
         /*  2*/ { "out_test_with_subdir", "out_test_subdir", "out.png", 1 },
@@ -348,7 +347,7 @@ static void test_fopen(const testCtx *const p_ctx) {
         /* 14*/ { "", "", "outé.png", 1 },
         /* 15*/ { "outé_test", "", "outé.png", 1 },
     };
-    int data_size = ARRAY_SIZE(data);
+    const int data_size = ARRAY_SIZE(data);
     int i, len;
 
     FILE *ret;
@@ -411,82 +410,6 @@ static void test_fopen(const testCtx *const p_ctx) {
     testFinish();
 }
 
-#ifndef _WIN32
-extern FILE *fmemopen(void *buf, size_t size, const char *mode);
-#endif
-
-static void test_out_putsf(const testCtx *const p_ctx) {
-    int debug = p_ctx->debug;
-
-    struct item {
-        const char *prefix;
-        int dp;
-        float arg;
-        const char *locale;
-        const char *expected;
-    };
-    /* s/\/\*[ 0-9]*\*\//\=printf("\/\*%3d*\/", line(".") - line("'<")): */
-    struct item data[] = {
-        /*  0*/ { "", 2, 1234.123, "", "1234.12" },
-        /*  1*/ { "", 3, 1234.123, "", "1234.123" },
-        /*  2*/ { "prefix ", 4, 1234.123, "", "prefix 1234.123" },
-        /*  3*/ { "", 2, -1234.126, "", "-1234.13" },
-        /*  4*/ { "", 2, 1234.1, "", "1234.1" },
-        /*  5*/ { "", 3, 1234.1, "", "1234.1" },
-        /*  6*/ { "", 4, 1234.1, "", "1234.1" },
-        /*  7*/ { "", 2, 1234.0, "", "1234" },
-        /*  8*/ { "", 2, -1234.0, "", "-1234" },
-        /*  9*/ { "", 3, 1234.1234, "de_DE.UTF-8", "1234.123" },
-        /* 10*/ { "", 4, -1234.1234, "de_DE.UTF-8", "-1234.1234" },
-        /* 11*/ { "prefix ", 4, -1234.1234, "de_DE.UTF-8", "prefix -1234.1234" },
-    };
-    int data_size = ARRAY_SIZE(data);
-    int i;
-
-    FILE *fp;
-    char buf[512] = {0}; /* Suppress clang-16/17 run-time exception MemorySanitizer: use-of-uninitialized-value */
-
-    testStart("test_out_putsf");
-
-#ifdef _WIN32
-    (void)i; (void)fp; (void)buf;
-    testSkip("Test not implemented on Windows");
-#else
-
-    for (i = 0; i < data_size; i++) {
-        const char *locale = NULL;
-
-        if (testContinue(p_ctx, i)) continue;
-
-        buf[0] = '\0';
-        fp = fmemopen(buf, sizeof(buf), "w");
-        assert_nonnull(fp, "%d: fmemopen fail (%d, %s)\n", i, errno, strerror(errno));
-
-        if (data[i].locale && data[i].locale[0]) {
-            locale = setlocale(LC_ALL, data[i].locale);
-            if (!locale) { /* May not be available - warn unless quiet mode */
-                if (!(debug & ZINT_DEBUG_TEST_LESS_NOISY)) {
-                    printf("%d: Warning: locale \"%s\" not available\n", i, data[i].locale);
-                }
-            }
-        }
-
-        out_putsf(data[i].prefix, data[i].dp, data[i].arg, fp);
-
-        assert_zero(fclose(fp), "%d: fclose fail (%d, %s)\n", i, errno, strerror(errno));
-
-        if (locale) {
-            assert_nonnull(setlocale(LC_ALL, locale), "%d: setlocale(%s) restore fail (%d, %s)\n",
-                i, locale, errno, strerror(errno));
-        }
-
-        assert_zero(strcmp(buf, data[i].expected), "%d: strcmp(%s, %s) != 0\n", i, buf, data[i].expected);
-    }
-
-    testFinish();
-#endif /* _WIN32 */
-}
-
 int main(int argc, char *argv[]) {
 
     testFunction funcs[] = { /* name, func */
@@ -496,7 +419,6 @@ int main(int argc, char *argv[]) {
         { "test_quiet_zones", test_quiet_zones },
         { "test_set_whitespace_offsets", test_set_whitespace_offsets },
         { "test_fopen", test_fopen },
-        { "test_out_putsf", test_out_putsf },
     };
 
     testRun(argc, argv, funcs, ARRAY_SIZE(funcs));
